<?php

class IntellectMoneyPayment {

    public $order_id;
    public $secret_key;
    public $eshop_id;
    public $order;
    public $recipient_currency;
    public $recipient_amount;
    public $preference;
    public $is_test;
    public $hold_mode;
    public $expire_date;
    public $hold_time;
    public $service_name;
    public $inn;
    public $tax;
    public $delivery_tax;
    public $group;
    public $lang;
    public $url;
    public $success_url;
    public $user_email;
    public $merchant_receipt;
    public $encoding;
    public $user_name;

    function __construct() {
        Bitrix\Main\Localization\Loc::loadMessages(__FILE__);
        $this->getSettings();
        $this->getOrder();
        $this->getCurrency();
        $this->getAmount();
        $this->getServiceName();
        $this->prepareHoldData();
        $this->generateHash();
        $this->generateMerchantReceipt();
        $this->getUrl();
        $this->getEncoding();
        $this->getUserName();
    }

    private function getSettings() {
        $CSalePaySystemAction = new CSalePaySystemAction();
        $this->eshop_id = $CSalePaySystemAction->GetParamValue("IM.ESHOP_ID");
        $this->secret_key = $CSalePaySystemAction->GetParamValue("IM.SEKRET_KEY");
        $this->order_id = $CSalePaySystemAction->GetParamValue("IM.ORDER_ID");
        $this->is_test = $CSalePaySystemAction->GetParamValue("IM.IS_TEST") == 'Y';
        $this->hold_mode = empty($CSalePaySystemAction->GetParamValue("IM.HOLD_MODE")) ? $CSalePaySystemAction->GetParamValue("IM.HOLD_MODE") : "NONE";
        $this->expire_date = $CSalePaySystemAction->GetParamValue("IM.EXPIRE_DATE");
        $this->hold_time = !$CSalePaySystemAction->GetParamValue("IM.HOLD_TIME");
        $this->preference = $CSalePaySystemAction->GetParamValue("IM.PREFERENCE");
        $this->lang = $CSalePaySystemAction->GetParamValue("IM.LANGUAGE");
        $this->tax = $CSalePaySystemAction->GetParamValue("IM.TAX");
        $this->inn = $CSalePaySystemAction->GetParamValue("IM.INN");
        $this->delivery_tax = $CSalePaySystemAction->GetParamValue("IM.DELIVERY_TAX");
        $this->group = !empty($CSalePaySystemAction->GetParamValue("IM.GROUP")) ? $CSalePaySystemAction->GetParamValue("IM.GROUP") : null;
        $this->success_url = $CSalePaySystemAction->GetParamValue("IM.SUCCESS_URL");
        $this->user_email = $CSalePaySystemAction->GetParamValue("IM.EMAIL");
    }

    private function getOrder() {
        $this->order = CSaleOrder::GetByID(IntVal($this->order_id));
    }
    private function getEncoding(){
        $this->encoding = CSalePaySystemAction::GetById($this->order["PAY_SYSTEM_ID"])["ENCODING"];
        $this->encoding = $this->encoding == "windows-1251" ? "cp1251" : "UTF-8";
    }

    private function getCurrency() {
        $this->recipient_currency = $this->is_test ? "TST" : $this->order["CURRENCY"];
    }

    private function getAmount() {
        $this->recipient_amount = number_format($this->order['PRICE'], 2, '.', '');
    }

    private function getServiceName() {
        $this->service_name = getMessage("IM.PAYMENT_PAYMENT_FOR_ORDER", array("#DATE#" => $this->order['DATE_INSERT_FORMAT'], "#ORDER_ID#" => $this->order_id));
    }

    private function prepareHoldData() {
        if($this->hold_mode == "1"){
            $this->expire_date = date('Y-m-d H:i:s', strtotime($this->order['DATE_INSERT'].'+' . $this->getHoldHours($this->expire_date) . ' hour'));
            $this->hold_time = $this->getHoldHours($this->hold_time);
        } elseif($this->hold_mode == "0") {
            $this->expire_date = date('Y-m-d H:i:s', strtotime($this->order['DATE_INSERT'].'+' . $this->getExpiredHours($this->expire_date) . ' hour'));
        }
    }

    private function getExpiredHours($days) {
        return (intval($days) && $days > 0 && $days < 4319) ? $days : 4319;
    }

    private function getHoldHours($hours) {
        return (intval($hours) && $hours > 0 && $hours < 119) ? $hours : 72;
    }

    private function generateMerchantReceipt() {
        $products = CSaleBasket::GetList(array(), array("ORDER_ID" => $this->order_id), false, false, array());
        $positions = [];
        while ($product = $products->Fetch()){
            $text = addslashes($product['NAME']);
            if($this->encoding != "UTF-8"){
                $text = mb_convert_encoding($text, "UTF-8", 'cp1251');
            }
            $positions[] = [
                'quantity' => $product['QUANTITY'],
                'price' => number_format($product['PRICE'], 2, '.', ''),
                'tax' => $this->tax,
                'text' => $text,
            ];
        }
        if ($this->order['PRICE_DELIVERY'] > 0){
            $text = "";
            if($this->encoding != "UTF-8"){
                $text = mb_convert_encoding($text, "UTF-8", 'cp1251');
            }
            $positions[] = [
                'quantity' => 1,
                'price' => number_format($this->order['PRICE_DELIVERY'], 2, '.', ''),
                'tax' => $this->delivery_tax,
                'text' => $text,
            ];
        }

        $merchantReceipt = [
            "inn" => $this->inn,
            "group" => $this->group,
            "content" => [
                "type" => "1",
                "positions" => $positions,
                "customerContact" => $this->user_email
            ]
        ];

        $this->merchant_receipt = json_encode($merchantReceipt);
    }

    private function generateHash() {
        $this->pre_hash = md5(join('::', array($this->eshop_id, $this->order_id, $this->service_name, $this->recipient_amount, $this->recipient_currency, $this->secret_key)));
    }

    private function getUrl() {
        $lang = empty($this->lang) ? 'ru' : strtolower($this->lang);
        $this->url = "https://merchant.intellectmoney.ru/" . $lang . "/";
    }
    
    private function getUserName(){
        $this->user_name = $this->order["USER_NAME"]." ".$this->order["USER_LAST_NAME"];
    }

    function getTemplate() {
        include '\..\template\template.php';
    }

}

class IntellectMoneyResult {

    public $module_id;
    public $order;
    public $order_id;
    public $secret_key;
    public $eshop_id;
    public $eshop_account;
    public $recipient_currency;
    public $recipient_amount;
    public $preference;
    public $is_test;
    public $is_hold;
    public $expire_date;
    public $hold_time;
    public $service_name;
    public $user_name;
    public $user_email;
    public $payment_data;
    public $group;
    public $lang;
    public $url;
    public $payment_status;
    public $statuses;

    function __construct() {
        define("STOP_STATISTICS", true);
        define("NOT_CHECK_PERMISSIONS", true);

        if (!require($_SERVER["DOCUMENT_ROOT"] . "/bitrix/modules/main/include/prolog_before.php"))
            die('prolog_before.php not found!');
        if (!CModule::IncludeModule("sale"))
            die('sale module not found');

        Bitrix\Main\Localization\Loc::loadMessages(__FILE__);
        $this->module_id = "im.payment";
        $this->getParams();
        CSalePaySystemAction::InitParamArrays($this->order, $this->order["ID"]);
    }

    private function getParams() {
        $this->eshop_id = $_REQUEST['eshopId'];
        $this->order_id = $_REQUEST['orderId'];
        $this->service_name = $_REQUEST['serviceName'];
        $this->eshop_account = $_REQUEST['eshopAccount'];
        $this->recipient_amount = number_format($_REQUEST['recipientAmount'], 2, '.', '');
        $this->recipient_currency = strtolower($_REQUEST['recipientCurrency']);
        $this->payment_status = $_REQUEST['paymentStatus'];
        $this->user_name = $_REQUEST['userName'];
        $this->user_email = $_REQUEST['userEmail'];
        $this->payment_data = $_REQUEST['paymentData'];
        $this->secret_key = $_REQUEST['secretKey'];
        $this->hash = $_REQUEST['hash'];
        $this->order = CSaleOrder::GetByID($this->order_id);
        $this->ar_fields_failed = array(
            "PS_STATUS" => "N",
            "STATUS_ID" => "N",
            "PS_RESPONSE_DATE" => Date(CDatabase::DateFormatToPHP(CLang::GetDateFormat("FULL", LANG))),
            "USER_ID" => $this->order["USER_ID"]
        );
        $this->statuses = array(
            "2" => 0,
            "3" => 1,
            "7" => 2,
            "6" => 3,
            "5" => 4,
            "4" => 5,
        );
    }

    function processingResponse() {
        if (!$this->checkIsDebug()) {
            ob_start();

            if (!$this->checkIP()) {
                $this->updateOrder(GetMessage("IM.PAYMENT_WRONG_IP", array("#ORDER_ID#" => $orderId)));
                $this->obExit("IP DID NOT MATCH");
            }

            if (!$this->issetOrder()) {
                $this->updateOrder(GetMessage("IM.PAYMENT_WRONG_ORDER_ID", array("#ORDER_ID#" => $orderId)));
                $this->obExit("ORDER NOT FOUND");
            }

            if (!$this->checkPaymentCurrency()) {
                $this->updateOrder(GetMessage("IM.PAYMENT_AMOUNT_CURRENCY_DONT_MATCH", array("#ORDER_ID#" => $orderId)));
                $this->obExit("CURRENCY DID NOT MATCH! response_currency: $this->recipient_currency; order_currency:" . $this->order['CURRENCY']);
            }

            if (!$this->checkEshopId()){
                $this->updateOrder(GetMessage("IM.PAYMENT_ESHOP_ID_DONT_MATCH", array("#ORDER_ID#" => $orderId)));
                $this->obExit("ERROR: INCORRECT ESHOP_ID! eshopId: $this->eshop_id; order_eshopId: " . CSalePaySystemAction::GetParamValue('IM.ESHOP_ID'));
            }

            if (!$this->checkSecretKey()) {
                $this->updateOrder(GetMessage("IM.PAYMENT_SECRET_KEY_DONT_MATCH", array("#ORDER_ID#" => $orderId)));
                $this->obExit("ERROR: SECRET_KEY MISMATCH!");
            }
            $this->checkHash();
            if ($this->isChangeOrder()){
                if ($this->checkPaymentAmount()){
                   $this->payment_status = '7';
                }
                if ($this->payment_status == '3') {
                    $arFieldsPending = array(
                        "STATUS_ID" => CSalePaySystemAction::GetParamValue("IM.PAYMENT_CREATE_STATUS"),
                        "PS_RESPONSE_DATE" => Date(CDatabase::DateFormatToPHP(CLang::GetDateFormat("FULL", LANG))),
                        "USER_ID" => $this->order["USER_ID"],
                        "PS_STATUS_CODE" => $this->payment_status
                    );
                    CSaleOrder::Update($this->order["ID"], $arFieldsPending);
                }
                if ($this->payment_status == '4') {
                    $arFieldsCanseled = array(
                        "PAYED" => "N",
                        "PS_STATUS" => "N",
                        "STATUS_ID" => CSalePaySystemAction::GetParamValue("IM.PAYMENT_CANSELED_STATUS"),
                        "PS_SUM" => $this->recipient_amount,
                        "CANCELED" => "Y",
                        "EMP_CANCELED_ID" => $this->order["USER_ID"],
                        "REASON_CANCELED" => GetMessage("IM.PAYMENT_CANSELED"),
                        "PS_STATUS_CODE" => $this->payment_status
                    );
                    CSaleOrder::CancelOrder($this->order["ID"], "Y");
                    CSaleOrder::Update($this->order["ID"], $arFieldsCanseled, GetMessage("IM.PAYMENT_CANSELED"));
                }
                if ($this->payment_status == '5') {
                    $arFieldsSuccess = array(
                        "PAYED" => "Y",
                        "PS_STATUS" => "Y",
                        "STATUS_ID" => CSalePaySystemAction::GetParamValue("IM.PAYMENT_PAYEED_STATUS"),
                        "PS_SUM" => $this->recipient_amount,
                        "PS_RESPONSE_DATE" => Date(CDatabase::DateFormatToPHP(CLang::GetDateFormat("FULL", LANG))),
                        "USER_ID" => $this->order["USER_ID"],
                        "PS_STATUS_DESCRIPTION" => GetMessage("IM.PAYMENT_PAYMENT_FOR_ORDER_SUCCESFUL", array("#ORDER_ID#" => $orderId)),
                        "PS_STATUS_CODE" => $this->payment_status
                    );
                    CSaleOrder::PayOrder($this->order["ID"], "Y");
                    CSaleOrder::Update($this->order["ID"], $arFieldsSuccess);
                }
                if ($this->payment_status == '6') {
                    $arFieldsHold = array(
                        "PAYED" => "Y",
                        "PS_STATUS" => "N",
                        "STATUS_ID" => CSalePaySystemAction::GetParamValue("IM.PAYMENT_HOLD_STATUS"),
                        "PS_SUM" => $this->recipient_amount,
                        "PS_RESPONSE_DATE" => Date(CDatabase::DateFormatToPHP(CLang::GetDateFormat("FULL", LANG))),
                        "USER_ID" => $this->order["USER_ID"],
                        "PS_STATUS_DESCRIPTION" => GetMessage("IM.PAYMENT_HOLD", array("#ORDER_ID#" => $orderId)),
                        "PS_STATUS_CODE" => $this->payment_status
                    );
                    CSaleOrder::Update($this->order["ID"], $arFieldsHold);
                }
                if($this->payment_status == '7'){
                    $arFieldsPurtiallyPay = array(
                        "PAYED" => "N",
                        "PS_STATUS" => "N",
                        "STATUS_ID" => CSalePaySystemAction::GetParamValue("IM.PAYMENT_PARTIALLY_PAYEED_STATUS"),
                        "SUM_PAID" => $this->recipient_amount,
                        "PS_RESPONSE_DATE" => Date(CDatabase::DateFormatToPHP(CLang::GetDateFormat("FULL", LANG))),
                        "USER_ID" => $this->order["USER_ID"],
                        "PS_STATUS_DESCRIPTION" => GetMessage("IM.PAYMENT_PARTIALLY_PAY", array("#ORDER_ID#" => $orderId)),
                        "PS_STATUS_CODE" => $this->payment_status
                    );
                    CSaleOrder::Update($this->order["ID"], $arFieldsPurtiallyPay);
                }
            }
            $this->obExit();
        }

        require($_SERVER["DOCUMENT_ROOT"] . "/bitrix/modules/main/include/epilog_after.php");
    }

    private function checkIsDebug() {
        if ($_REQUEST['debug_file']) {
            if ($this->checkIP()) {
                $this->showFile();
                return true;
            }
        }
        return false;
    }

    private function showFile() {
        header('Content-type: text/plain; charset=utf-8');
        echo file_get_contents(__FILE__);
    }

    private function checkIP() {
        return in_array($_SERVER['REMOTE_ADDR'], array("139.45.224.254", "127.0.0.1"));
    }

    private function checkPaymentAmount() {
        return number_format($this->order["PRICE"], 2, '.', '') > $this->recipient_amount+$this->order["SUM_PAID"];
    }

    private function checkPaymentCurrency() {
        $order_currency = strtolower($this->order["CURRENCY"]);
        $order_currency = ($order_currency == 'rur') ? 'rub' : $order_currency;
        return !($this->recipient_currency != 'tst' && $order_currency != $this->recipient_currency);
    }

    private function updateOrder($message) {
        $this->arFieldsFailed["PS_STATUS_DESCRIPTION"] = $message;
        CSaleOrder::Update($this->order["ID"], $this->ar_fields_failed);
    }

    private function issetOrder() {
        if (empty($this->order)) {
            AddMessage2Log(GetMessage("IM.PAYMENT_WRONG_ORDER_ID", array("#ORDER_ID#" => $orderId)), $this->module_id);
            SendError(GetMessage("IM.PAYMENT_WRONG_ORDER_ID", array("#ORDER_ID#" => $orderId)), $this->module_id);
            return false;
        }
        return true;
    }
    
    private function checkEshopId() {
        return ($this->eshop_id == CSalePaySystemAction::GetParamValue('IM.ESHOP_ID'));
    }

    private function checkSecretKey() {
        return !($this->secret_key && $this->secret_key != CSalePaySystemAction::GetParamValue("IM.SEKRET_KEY"));
    }

    private function checkHash() {
        $control_hash_str = implode('::', array(
            $this->eshop_id, $this->order_id, $this->service_name,
            $this->eshop_account, $this->recipient_amount, strtoupper($this->recipient_currency), $this->payment_status,
            $this->user_name, $this->user_email, $this->payment_data, CSalePaySystemAction::GetParamValue("IM.SEKRET_KEY")
        ));

        $control_hash = md5($control_hash_str);
        $control_hash_utf8 = md5(iconv('windows-1251', 'utf-8', $control_hash_str));
        if (($this->hash != $control_hash && $this->hash != $control_hash_utf8) || !$this->hash){
            $this->updateOrder(GetMessage("IM.PAYMENT_SIGNS_DONT_MATCH", array("#ORDER_ID#" => $orderId)));
            $err = "ERROR: HASH MISMATCH\n";
            $err .= $this->checkIP() ? "Control hash string: $control_hash_str;\n" : "";
            $err .= "Control hash win-1251: $control_hash;\nControl hash utf-8: $control_hash_utf8;\nhash: $this->hash;\n\n";
            $this->obExit($err);
        }
    }

    private function getPaymentStatus(){
        $dbOrders = \Bitrix\Sale\Internals\PaymentTable::getList(array(
            'filter' => array('ORDER_ID' => $this->order_id)
        ));
        $payment_status = "2";
        while($order = $dbOrders->fetch()){
            if ($this->statuses[$payment_status] < $this->statuses[$order["PS_STATUS_CODE"]]){
                $payment_status = $order["PS_STATUS_CODE"];
            }
        }
        return $payment_status;
    }

    private function isChangeOrder(){
        return $this->statuses[$this->getPaymentStatus()] <= $this->statuses[$this->payment_status];
    }

    private function obExit($status = null) {
        if ($status) {
            ob_end_flush();
            exit($status);
        } else {
            ob_end_clean();
            header("HTTP/1.0 200 OK");
            echo "OK";
            exit();
        }
    }

}
